create or replace function pgq.create_queue(
    i_queue_name text,
    i_rotation_delay integer,
    i_event_timeout integer,
    i_numtables integer,
    i_batch_log_timeout interval,
    i_use_batch_log boolean)
returns integer as $$
declare
    tblpfx   text;
    tblname  text;
    idxpfx   text;
    idxname  text;
    sql      text;
    id       integer;
    v_rotation_delay interval;
    x_queue_id integer;
    tick_seq text;
    ev_seq text;
begin
    if i_queue_name is null or i_numtables is null then
        raise exception 'Invalid NULL value';
    end if;
    if i_numtables < 3 then
        raise exception 'Too small number of tables';
    end if;

    -- check if exists
    select table_name into tblpfx from pgq.queue_config
        where queue_name = i_queue_name;
    if found then
        return 0;
    end if;

    -- calc final values
    if i_rotation_delay > 0 then
        v_rotation_delay := (i_rotation_delay || ' seconds')::interval;
    else
        v_rotation_delay := null;
    end if;
    if i_event_timeout > 0 then
        raise exception 'Event timeout not supported';
    end if;

    -- insert event
    id := nextval('pgq.queue_config_queue_id_seq');
    tblpfx := 'pgq.event_' || id;
    idxpfx := 'event_' || id;
    tick_seq := 'pgq.event_' || id || '_tick_seq';
    ev_seq := 'pgq.event_' || id || '_id_seq';
    insert into pgq.queue_config (queue_id, queue_name,
            table_name, number_of_event_tables, active_event_table,
            rotation_delay, batch_log_timeout, use_batch_log,
            tick_seq_name, event_seq_name)
        values (id, i_queue_name, tblpfx, i_numtables, 0,
            v_rotation_delay, i_batch_log_timeout, i_use_batch_log,
            tick_seq, ev_seq);

    -- create seqs
    execute 'CREATE SEQUENCE ' || tick_seq;
    execute 'CREATE SEQUENCE ' || ev_seq;

    -- create data tables
    execute 'CREATE TABLE ' || tblpfx || ' () '
            || ' INHERITS (pgq.event_template)';
    for i in 0 .. (i_numtables - 1) loop
        tblname := tblpfx || '_' || i;
        idxname := idxpfx || '_' || i;
        execute 'CREATE TABLE ' || tblname || ' () '
                || ' INHERITS (' || tblpfx || ')';
        execute 'ALTER TABLE ' || tblname || ' ALTER COLUMN event_id '
                || ' SET DEFAULT nextval(' || quote_literal(ev_seq) || ')';
        execute 'create index ' || idxname || '_txid_idx on '
                || tblname || ' (event_txid)';
    end loop;

    perform pgq.grant_perms(i_queue_name);

    perform pgq.ticker(id);

    return 1;
end;
$$ language plpgsql security definer;

-- use batch log by default
create or replace function pgq.create_queue(
    i_queue_name text,
    i_rotation_delay integer,
    i_event_timeout integer,
    i_numtables integer)
returns integer as $$
begin
    return pgq.create_queue(i_queue_name, i_rotation_delay,
                i_event_timeout, i_numtables,
                '10 minutes', true);
end;
$$ language plpgsql;

-- default number of tables
create or replace function pgq.create_queue(
    i_queue_name text,
    i_rotation_delay integer,
    i_event_timeout integer)
returns integer as $$
begin
    return pgq.create_queue(i_queue_name,
                            i_rotation_delay, i_event_timeout,
                            3);
end;
$$ language plpgsql; -- no perms needed

-- default rotation, no event timeout
create or replace function pgq.create_queue(
    i_queue_name text)
returns integer as $$
begin
    return pgq.create_queue(i_queue_name, 7200, 0);
end;
$$ language plpgsql; -- no perms needed


