-- extended version, allow to register on particular tick
create or replace function pgq.register_consumer(
    x_queue_name text,
    x_consumer_name text,
    x_thread_timeout integer,
    x_tick_pos bigint)
returns integer as $$
declare
    tmp         text;
    last_tick   bigint;
    timeout     interval;
    cur_timeout interval;
    cf          record;
    x_consumer_id integer;
begin
    select queue_id into cf
        from pgq.queue_config
        where queue_name = x_queue_name;
    if not found then
        raise exception 'Event queue not created yet';
    end if;

    -- lock consumer and create if new
    select consumer_id into x_consumer_id
        from pgq.consumer
        where consumer_name = x_consumer_name
        for update;
    if not found then
        insert into pgq.consumer (consumer_name) values (x_consumer_name);
        x_consumer_id := currval('pgq.consumer_consumer_id_seq');
    end if;

    -- calc timeout
    if x_thread_timeout > 0 then
        timeout := (x_thread_timeout || ' seconds') :: interval;
    else
        timeout := NULL;
    end if;

    -- if particular tick was requested, check if it exists
    if x_tick_pos is not null then
        perform 1 from pgq.tick
            where queue_id = cf.queue_id
              and tick_id = x_tick_pos;
        if not found then
            raise exception 'cannot reposition, tick not found: %', x_tick_pos;
        end if;
    end if;

    -- check if double registration
    select last_tick_id, thread_timeout
        into last_tick, cur_timeout
        from pgq.high_watermark
        where consumer_id = x_consumer_id
          and queue_id = cf.queue_id;
    if found then
        if x_tick_pos is not null then
            -- update tick pos if requested
            update pgq.high_watermark
                set last_tick_id = x_tick_pos
                where consumer_id = x_consumer_id
                  and queue_id = cf.queue_id;
        end if;
        -- already registered
        return 0;
    end if;

    --  new registration
    if x_tick_pos is null then
        -- start from current tick
        select tick_id into last_tick from pgq.tick
            where queue_id = cf.queue_id
            order by queue_id desc, tick_id desc
            limit 1;
        if not found then
            raise exception 'No ticks for this queue.  Please run ticker on database.';
        end if;
    else
        last_tick := x_tick_pos;
    end if;

    -- register
    insert into pgq.high_watermark
        (consumer_id, last_tick_id, queue_id, thread_timeout)
        values (x_consumer_id, last_tick, cf.queue_id, timeout);
    return 1;
end;
$$ language plpgsql security definer;

-- standard version - register to current position
create or replace function pgq.register_consumer(
    x_queue_name text,
    x_consumer_id text,
    x_thread_timeout integer)
returns integer as $$
begin
    return pgq.register_consumer(x_queue_name, x_consumer_id, x_thread_timeout, null);
end;
$$ language plpgsql; -- no perms needed

-- version with default thread_timeout
create or replace function pgq.register_consumer(
    x_queue_name text,
    x_consumer_id text)
returns integer as $$
-- default = 2h
begin
    return pgq.register_consumer(x_queue_name, x_consumer_id, 7200);
end;
$$ language plpgsql; -- no perms needed

